@@ -2,13 +2,13 @@ import requests
 import time
 import aiohttp
 import asyncio
-import tempfile
 import hashlib
 import os
 import base64
 
-from .config import create_task_url, get_result_url, app_key
-from .errors import ParamError, DownloadError, ReadError, IdGetError
+from .config import create_task_url, app_key
+from .errors import ParamError, ReadError, IdGetError
+from .get_answer import get_sync_result, get_async_result
 
 
 class ImageToTextTask:
@@ -17,7 +17,7 @@ class ImageToTextTask:
     Подробней информацию смотрите в методе 'captcha_handler' и '__init__'
     '''
 
-    def __init__(self, anticaptcha_key, sleep_time=5, save_format = 'temp', language = 'en',**kwargs):
+    def __init__(self, anticaptcha_key: str, sleep_time: int = 5, save_format: str = 'temp', language: str = 'en', callbackUrl: str = None, **kwargs):
         '''
         Инициализация нужных переменных, создание папки для изображений и кэша
         После завершения работы - удаляются временные фалйы и папки
@@ -26,6 +26,7 @@ class ImageToTextTask:
         :param save_format: Формат в котором будет сохраняться изображение, либо как временный файл - 'temp',
                             либо как обычное изображение в папку созданную библиотекой - 'const'.
         :param language: Язык капчи
+        :param callbackUrl: URL для решения капчи с ответом через callback
         :param **kwargs: За подробной информацией обратитесь к документации на сайте anticaptcha.
         '''
         if sleep_time < 5:
@@ -49,6 +50,10 @@ class ImageToTextTask:
                              "softId": app_key
                              }
 
+        # задаём callbackUrl если передан
+        if callbackUrl:
+            self.task_payload.update({'callbackUrl': callbackUrl})
+
         # отправляем запрос на результат решения капчи, если ещё капча не решена - ожидаем 5 сек
         # если всё ок - идём дальше
         self.result_payload = {"clientKey": anticaptcha_key}
@@ -58,21 +63,19 @@ class ImageToTextTask:
             for key in kwargs:
                 self.task_payload['task'].update({key: kwargs[key]})
 
-    def image_temp_saver(self, content):
+    def image_temp_saver(self, content: bytes):
         '''
         Метод сохраняет файл изображения как временный и отправляет его сразу на сервер для расшифровки.
         :return: Возвращает ID капчи
         '''
-        with tempfile.NamedTemporaryFile(suffix='.png', delete=False) as captcha_image:
-            captcha_image.write(content)
-            # Создаём пайлоад, вводим ключ от сайта, выбираем метод ПОСТ и ждём ответа в JSON-формате
-            self.task_payload['task'].update({"body": base64.b64encode(captcha_image.read()).decode('utf-8')})
-            # Отправляем на рукапча изображение капчи и другие парметры,
-            # в результате получаем JSON ответ с номером решаемой капчи и получая ответ - извлекаем номер
-            captcha_id = (requests.post(create_task_url, json = self.task_payload).json())
+        # Создаём пайлоад, вводим ключ от сайта, выбираем метод ПОСТ и ждём ответа в JSON-формате
+        self.task_payload['task'].update({"body": base64.b64encode(content).decode('utf-8')})
+        # Отправляем на рукапча изображение капчи и другие парметры,
+        # в результате получаем JSON ответ с номером решаемой капчи и получая ответ - извлекаем номер
+        captcha_id = requests.post(create_task_url, json = self.task_payload).json()
         return captcha_id
 
-    def image_const_saver(self, content):
+    def image_const_saver(self, content: bytes):
         '''
         Метод создаёт папку и сохраняет в неё изображение, затем передаёт его на расшифровку и удалет файл.
         :return: Возвращает ID капчи
@@ -100,7 +103,7 @@ class ImageToTextTask:
 
         return captcha_id
     
-    def read_captcha_image_file(self, content, content_type="file"):
+    def read_captcha_image_file(self, content: bytes, content_type: str = "file"):
         """
         Функция отвечает за чтение уже сохранённого файла или файла в уодировке base64
         :param content: Параметр строка-путь указывающий на изображение капчи для отправки её на сервер
@@ -128,7 +131,7 @@ class ImageToTextTask:
         return captcha_id
     
     # Работа с капчёй
-    def captcha_handler(self, captcha_link=None, captcha_file=None, captcha_base64=None, **kwargs):
+    def captcha_handler(self, captcha_link: str = None, captcha_file: str = None, captcha_base64: str = None, **kwargs):
         '''
         Метод получает от вас ссылку на изображение, скачивает его, отправляет изображение на сервер
         RuCaptcha, дожидается решения капчи и вовзращает вам результат
@@ -157,26 +160,16 @@ class ImageToTextTask:
             captcha_id = captcha_id["taskId"]
             self.result_payload.update({"taskId": captcha_id})
         else:
-            raise IdGetError(server_answer=captcha_id)
-
-        # Ожидаем решения капчи
-        time.sleep(self.sleep_time)
-        while True:
-            # отправляем запрос на результат решения капчи, если не решена ожидаем
-            captcha_response = requests.post(get_result_url, json = self.result_payload)
-
-            # Если ошибки нет - проверяем статус капчи
-            if captcha_response.json()['errorId'] == 0:
-                # Если капча ещё не готова- ожидаем
-                if captcha_response.json()["status"] == "processing":
-                    time.sleep(self.sleep_time)
-                # если уже решена - возвращаем ответ сервера
-                else:
-                    return captcha_response.json()
-            # Иначе возвращаем ответ сервера
-            else:
-                return captcha_response.json()
+            return captcha_id
 
+        # если передан параметр `callbackUrl` - не ждём решения капчи а возвращаем незаполненный ответ
+        if self.task_payload.get('callbackUrl'):
+            return self.result_payload
+            
+        else:
+            # Ожидаем решения капчи
+            time.sleep(self.sleep_time)
+            return get_sync_result(result_payload = self.result_payload, sleep_time = self.sleep_time)
 
 
 class aioImageToTextTask:
@@ -185,7 +178,7 @@ class aioImageToTextTask:
 	Подробней информацию смотрите в методе 'captcha_handler' и '__init__'
 	'''
     
-    def __init__(self, anticaptcha_key, sleep_time=5, save_format='temp', language='en', **kwargs):
+    def __init__(self, anticaptcha_key: str, sleep_time: int = 5, save_format: str = 'temp', language: str = 'en', callbackUrl: str = None, **kwargs):
         '''
 		Инициализация нужных переменных, создание папки для изображений и кэша
 		После завершения работы - удаляются временные фалйы и папки
@@ -194,6 +187,7 @@ class aioImageToTextTask:
 		:param save_format: Формат в котором будет сохраняться изображение, либо как временный файл - 'temp',
 							либо как обычное изображение в папку созданную библиотекой - 'const'.
 		:param language: Язык капчи
+		:param callbackUrl: URL для решения капчи с ответом через callback
 		:param **kwargs: За подробной информацией обратитесь к документации на сайте anticaptcha.
 		'''
         if sleep_time < 5:
@@ -218,6 +212,10 @@ class aioImageToTextTask:
                              "softId": app_key
                              }
         
+        # задаём callbackUrl если передан
+        if callbackUrl:
+            self.task_payload.update({'callbackUrl': callbackUrl})
+
         # отправляем запрос на результат решения капчи, если ещё капча не решена - ожидаем 5 сек
         # если всё ок - идём дальше
         self.result_payload = {"clientKey": anticaptcha_key}
@@ -227,7 +225,7 @@ class aioImageToTextTask:
             for key in kwargs:
                 self.task_payload['task'].update({key: kwargs[key]})
     
-    async def image_temp_saver(self, captcha_link):
+    async def image_temp_saver(self, captcha_link: str):
         '''
 		Метод сохраняет файл изображения как временный и отправляет его сразу на сервер для расшифровки.
 		:return: Возвращает ID капчи
@@ -236,18 +234,15 @@ class aioImageToTextTask:
         async with aiohttp.ClientSession() as session:
             async with session.get(captcha_link) as resp:
                 content = await resp.content.readany()
-        
-        with tempfile.NamedTemporaryFile(suffix='.png') as captcha_image:
-            captcha_image.write(content)
-            # Создаём пайлоад, вводим ключ от сайта, выбираем метод ПОСТ и ждём ответа в JSON-формате
-            self.task_payload['task'].update({"body": base64.b64encode(captcha_image.read()).decode('utf-8')})
-            # Отправляем на рукапча изображение капчи и другие парметры,
-            # в результате получаем JSON ответ с номером решаемой капчи
-            async with aiohttp.ClientSession() as session:
-                async with session.post(create_task_url, json=self.task_payload) as resp:
-                    return await resp.json()
+                # Создаём пайлоад, вводим ключ от сайта, выбираем метод ПОСТ и ждём ответа в JSON-формате
+                self.task_payload['task'].update({"body": base64.b64encode(content).decode('utf-8')})
+                # Отправляем на рукапча изображение капчи и другие парметры,
+                # в результате получаем JSON ответ с номером решаемой капчи
+        async with aiohttp.ClientSession() as session:
+            async with session.post(create_task_url, json=self.task_payload) as resp:
+                return await resp.json()
     
-    async def image_const_saver(self, captcha_link):
+    async def image_const_saver(self, captcha_link: str):
         '''
 		Метод создаёт папку и сохраняет в неё изображение, затем передаёт его на расшифровку и удалет файл.
 		:return: Возвращает ID капчи
@@ -282,7 +277,7 @@ class aioImageToTextTask:
         os.remove(os.path.join(img_path, "im-{0}.png".format(image_hash)))
         return captcha_id
 
-    async def read_captcha_image_file(self, content, content_type='file'):
+    async def read_captcha_image_file(self, content: bytes, content_type: str = 'file'):
         """
         Функция отвечает за чтение уже сохранённого файла или файла в уодировке base64
         :param content: Параметр строка-путь указывающий на изображение капчи для отправки её на сервер
@@ -310,7 +305,7 @@ class aioImageToTextTask:
         return captcha_id
 
     # Работа с капчёй
-    async def captcha_handler(self, captcha_link = None, captcha_file = None, captcha_base64=None):
+    async def captcha_handler(self, captcha_link: str = None, captcha_file: str = None, captcha_base64: str=None):
         '''
 		Метод получает от вас ссылку на изображение, скачивает его, отправляет изображение на сервер
 		RuCaptcha, дожидается решения капчи и вовзращает вам результат
@@ -319,9 +314,9 @@ class aioImageToTextTask:
 		'''
         # если был передан линк на локальный скачаный файл
         if captcha_file:
-            captcha_id = self.read_captcha_image_file(captcha_file, content_type="file")
+            captcha_id = await self.read_captcha_image_file(captcha_file, content_type="file")
         elif captcha_base64:
-            captcha_id = self.read_captcha_image_file(captcha_base64, content_type="base64")
+            captcha_id = await self.read_captcha_image_file(captcha_base64, content_type="base64")
         elif captcha_link:
             # согласно значения переданного параметра выбираем функцию для сохранения изображения
             if self.save_format == 'const':
@@ -337,25 +332,13 @@ class aioImageToTextTask:
             captcha_id = captcha_id["taskId"]
             self.result_payload.update({"taskId": captcha_id})
         else:
-            raise IdGetError(server_answer=captcha_id)
-
-        # Ожидаем решения капчи
-        await asyncio.sleep(self.sleep_time)
-        # отправляем запрос на результат решения капчи, если не решена ожидаем
-        async with aiohttp.ClientSession() as session:
-            while True:
-                async with session.post(get_result_url, json=self.result_payload) as resp:
-                    json_result = await resp.json()
-                    
-                    # Если ошибки нет - проверяем статус капчи
-                    if json_result['errorId'] == 0:
-                        # Если капча ещё не готова- ожидаем
-                        if json_result["status"] == "processing":
-                            await asyncio.sleep(self.sleep_time)
-                        # если уже решена - возвращаем ответ сервера
-                        else:
-                            return json_result
-                    # Иначе возвращаем ответ сервера
-                    else:
-                        return json_result
+            return captcha_id
 
+        # если передан параметр `callbackUrl` - не ждём решения капчи а возвращаем незаполненный ответ
+        if self.task_payload.get('callbackUrl'):
+            return self.result_payload
+            
+        else:
+            # Ждем решения капчи
+            await asyncio.sleep(self.sleep_time)
+            return await get_async_result(result_payload = self.result_payload, sleep_time = self.sleep_time)
